package	jp.co.sds_corp.util.daemon.file;

import java.io.File;
import java.util.Date;

import jp.co.sds_corp.util.config.IConfiguration;

/**
最終更新時刻によるファイル名フィルタです。
<P>
XMLは以下の形式です。
<PRE>
&lt;Filter class="jp.co.sds_corp.util.daemon.file.LastModifiedFilter"&gt;
	&lt;term type="date"&gt;10&lt;/term&gt;
&lt;/Filter&gt;
</PRE>
期間タイプは"date"と"hour"の指定が可能です。
include="true"を指定すると、指定期間内のファイルが選択されます。
<BR>
上記の例では、最終更新時間から10日以上経過したファイルが選択されます。
*/
public class LastModifiedFilter implements IFilenameFilter
{
	public enum TermType {
		DATE, HOUR;
	};
	TermType	type;	// 期間タイプ
	int	term;	// 期間
	boolean	include;	// include


	/**
	* 設定情報から期間の情報を取得し、初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		IConfiguration c;
		c = config.getChild( "term" );
		String  var;
		var = c.getAttributes().get( "type" );
		if ( var.equals( "date" )) {
			type = TermType.DATE;
		} else
		if ( var.equals( "hour" )) {
			type = TermType.HOUR;
		} else {
			throw( new IllegalArgumentException( "illegal type [" + var + "]" ));
		}
		term = Integer.valueOf( c.get() ).intValue();
		var = c.getAttributes().get( "include", "" );
		if ( var.equals( "true" )) {
			include = true;
		} else {
			include = false;
		}
	}

	/**
	* 指定されたファイルをファイルリストに含めるかどうかをテストします。
	* @param	path	ファイルが見つかったディレクトリ
	* @param	name	ファイルの名前
	*/
	public boolean accept( File path, String name ) {
		long now = new Date().getTime();
		File file = new File( path, name );
		long last = file.lastModified();
		long limit = 0;
		switch ( type ) {
		case DATE:
			limit = 24L * 60L * 60L * 1000L * term;
			break;
		case HOUR:
			limit = 60L * 60L * 1000L * term;
		}
		boolean bRet;
		if (( now - last ) > limit ) {
			bRet = true;
		} else {
			bRet = false;
		}
		if ( include ) {
			bRet = !bRet;
		}
		return bRet;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		config.setAttribute( "class", getClass().getName() );
		IConfiguration c = config.addChild( "term", String.valueOf( term ));
		String t = null;
		switch ( type ) {
		case DATE: t = "date"; break;
		case HOUR: t = "hour"; break;
		}
		c.setAttribute( "type", t );
	}

	/**
	* このファイル名フィルタの文字列表現を返します。
	* @return	このファイル名フィルタの文字列表現
	*/
	public String toString() {
		String ret = getClass().getName();
		String t = null;
		switch ( type ) {
		case DATE: t = "date"; break;
		case HOUR: t = "hour"; break;
		}
		ret += ":type=" + t + " term=" + term;
		return ret;
	}

	/**
	* 期間タイプを設定します。
	* @param	var		設定するデータ
	*/
	public void setType( TermType var ) {
		this.type = var;
	}
	/**
	* 期間タイプを取得します。
	* @return	期間タイプ
	*/
	public TermType getType() {
		return type;
	}

	/**
	* 期間を設定します。
	* @param	var		設定するデータ
	*/
	public void setTerm( int var ) {
		this.term = var;
	}
	/**
	* 期間を取得します。
	* @return	期間
	*/
	public int getTerm() {
		return term;
	}

	/**
	* includeを設定します。
	* @param	var		設定するデータ
	*/
	public void setInclude( boolean var ) {
		this.include = var;
	}
	/**
	* includeを取得します。
	* @return	include
	*/
	public boolean getInclude() {
		return include;
	}


}
