package	jp.co.sds_corp.util.daemon.remote;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.reflect.Method;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.IInitializer;
import jp.co.sds_corp.util.daemon.IService;
import jp.co.sds_corp.util.daemon.core.CronDaemon;
import jp.co.sds_corp.util.daemon.core.IDaemon;
import jp.co.sds_corp.util.daemon.core.Interval;
import jp.co.sds_corp.util.daemon.core.Schedule;
import jp.co.sds_corp.util.daemon.core.TimerDaemon;
import jp.co.sds_corp.util.daemon.manager.DaemonManager;
import jp.co.sds_corp.util.thread.ThreadMonitor;

/**
* デーモンマネージャを外部から制御します。
*/
public class Commander
{
	DaemonManager	manager;

	public Commander( DaemonManager manager ) {
		this.manager = manager;
	}

	/**
	* 全てのデーモンを開始します。
	*/
	public void start() {
		manager.start();
	}
	/**
	* 指定された名前のデーモンを開始します。
	* @param	name	デーモンの名前
	*/
	public void start( String name ) {
		IDaemon dmn = manager.getDaemon( name );
		if ( dmn == null ) {
			throw( new IllegalArgumentException( "Daemon[" + name + "]"
				+ " not found." ));
		}
		if ( !dmn.isAlive() ) {
			dmn.start();
		}
	}

	/**
	* 全てのデーモンを停止します。
	*/
	public void stop() {
		manager.stop();
	}
	/**
	* 指定された名前のデーモンを停止します。
	* @param	name	デーモンの名前
	*/
	public void stop( String name ) {
		IDaemon dmn = manager.getDaemon( name );
		if ( dmn == null ) {
			throw( new IllegalArgumentException( "Daemon[" + name + "]"
				+ " not found." ));
		}
		if ( dmn.isAlive() ) {
			dmn.terminate();
			dmn.joinAll();
		}
	}

	/**
	* 全てのデーモンを停止して、デーモンマネージャーを終了します。
	*/
	public void quit() {
		manager.quit();
	}

	/**
	* 全てのデーモンの情報を再読み込みします。
	*/
	public void reload() throws Exception {
		manager.stop();
		manager.init();
	}

	/**
	* 全てのデーモンを初期化します。
	*/
	public void initialize() throws Exception {
		Iterator<String> itr = manager.getInitializerMap().keySet().iterator();
		while ( itr.hasNext()) {
			IInitializer i = manager.getInitializer( (String )itr.next() );
			i.start();
		}
	}
	/**
	* 指定された名前のデーモンを初期化します。
	* @param	name	デーモンの名前
	*/
	public void initialize( String name ) throws Exception {
		IInitializer i = manager.getInitializer( name );
		i.start();
	}

	/**
	* 全てのデーモンの情報を文字列で取得します。
	*/
	public String status() {
		String res = "";
		Iterator<String> itr = manager.getDaemonMap().keySet().iterator();
		while ( itr.hasNext()) {
			IDaemon dmn = manager.getDaemon( (String )itr.next() );
			res += dmn + "\n:" + dmn.getStatusString() + "\n";
		}
		return res;
	}
	/**
	* 指定された名前のデーモンの情報を文字列で取得します。
	* @param	name	デーモンの名前
	*/
	public String status( String name ) {
		String res = "";
		IDaemon dmn = manager.getDaemon( name );
		if ( dmn == null ) {
			throw( new IllegalArgumentException( "Daemon[" + name + "]"
				+ " not found." ));
		}
		res = dmn + "\n:" + dmn.getStatusString() + "\n";
		return res;
	}

	/**
	* Threadの情報を表示します。
	*/
	public String thread() {
		ThreadMonitor mon = new ThreadMonitor();
		return mon.getString();
	}
	/**
	* JVMの情報を表示します。
	*/
	public String vminfo() {
		DecimalFormat df = new DecimalFormat( "#,###" );
		Runtime rt = Runtime.getRuntime();
		String ret = "";
		ret += "--- JVM Info ---\n";
		ret += System.getProperty( "java.vm.name" ) + " ";
		ret += System.getProperty( "java.vm.version" ) + "\n";
		ret += "--- Memory ---\n";
		ret += "max memory=" + df.format( rt.maxMemory() ) + "\n";
		ret += "total memory=" + df.format( rt.totalMemory() ) + "\n";
		ret += "free memory=" + df.format( rt.freeMemory() ) + "\n";

		return ret;
	}
	/**
	* JVMの情報を表示します。
	*/
	public String properties() {
		StringWriter sw = new StringWriter();
		System.getProperties().list( new PrintWriter( sw ));
		String ret = "";
		ret += "--- Properties ---\n";
		ret += sw.getBuffer().toString();
		return ret;
	}
	/**
	* Garbage Collectionを実施します。
	*/
	public void gc() {
		System.gc();
	}

	/**
	* 全てのデーモンの情報を文字列で取得します。
	* @param	file	出力するファイル名
	*/
	public void save( String file, String name ) throws Exception {
		manager.save( file, name );
	}

	/**
	* サービスの一覧を返します。
	* @return	サービスの一覧
	*/
	public Map<String,IService> getServiceMap() throws Exception {
		HashMap<String,IService> ret = new HashMap<String,IService>();
		Map<String,IDaemon> map = manager.getDaemonMap();
		Iterator<String> itr = map.keySet().iterator();
		while ( itr.hasNext() ) {
			String key = (String )itr.next();
			IDaemon dmn = (IDaemon )map.get( key );
			IService svc = dmn.getService();
			ret.put( svc.getName(), svc );
		}
		return ret;
	}

	/**
	* 指定された名前のサービスを返します。
	* @param	name	サービスの名前
	* @return	指定されたサービス
	*/
	public IService getService( String name ) throws Exception {
		Map<String,IService> map = getServiceMap();
		IService svc = (IService )map.get( name );
		return svc;
	}

	/**
	* 指定された名前のサービスの状態を返します。
	* @param	name	サービスの名前
	* @return	指定されたサービスの状態
	*/
	public IService.Status getServiceStatus( String name ) throws Exception {
		IService svc = getService( name );
		if ( svc == null ) {
			return IService.Status.UNKNOWN;
		}
		return svc.getStatus();
	}

	/**
	* デーモンの一覧を返します。
	* @return	デーモンの一覧
	*/
	public Map<String,IDaemon> getDaemonMap() throws Exception {
		return manager.getDaemonMap();
	}

	/**
	* 指定された名前のデーモンを返します。
	* @param	name	デーモンの名前
	* @return	指定されたデーモン
	*/
	public IDaemon getDaemon( String name ) throws Exception {
		return manager.getDaemon( name );
	}

	/**
	* 指定された名前のデーモンの状態を返します。
	* @param	name	デーモンの名前
	* @return	指定されたデーモンの状態
	*/
	public IDaemon.Status getDaemonStatus( String name ) throws Exception {
		IDaemon dmn = manager.getDaemon( name );
		if ( dmn == null ) {
			return IDaemon.Status.UNKNOWN;
		}
		return dmn.getStatus();
	}

	/**
	* 指定された名前のデーモンを削除します。
	* @param	name	デーモンの名前
	*/
	public void removeDaemon( String name ) throws Exception {
		manager.removeDaemon( name );
	}

	/**
	* 設定情報で初期化されるデーモンを追加します。
	* @param	config	設定情報
	*/
	public IDaemon addDaemon( IConfiguration config ) throws Exception {
		IDaemon dmn = manager.createDaemon( config, 0 );
		manager.getDaemonMap().put( dmn.getName(), dmn );
		return dmn;
	}

	/**
	* 指定された名前のデーモンを削除します。
	* @param	name	デーモンの名前
	* @param	method	methodの名前
	* @return	指定されたデーモン
	*/
	public Object invokeDaemon( String name, String method ) throws Exception {
		Class<?>[] cls = {};
		Object[] prm = {};
		IDaemon dmn = manager.getDaemon( name );
		if ( dmn == null ) {
			throw( new IllegalArgumentException( "Daemon[" + name + "]"
				+ " not found." ));
		}
		Method mtd = dmn.getClass().getMethod( method, cls );
		Object ret = mtd.invoke( dmn, prm );
		return ret;
	}

	/**
	* CronDaemonにスケジュールを追加します。
	* @param	name	デーモンの名前
	* @param	sch		スケジュール
	*/
	public void addSchedule( String name, Schedule sch ) throws Exception {
		IDaemon dmn = manager.getDaemon( name );
		if ( dmn == null ) {
			throw( new IllegalArgumentException( "[" + name + "]"
				+ " not found." ));
		}
		if ( ! ( dmn instanceof CronDaemon )) {
			throw( new IllegalArgumentException( "[" + name + "]"
				+ " is not CronDaemon" ));
		}
		((CronDaemon)dmn).addSchedule( sch );
	}

	/**
	* CronDaemonのスケジュールを削除します。
	* @param	name	デーモンの名前
	* @param	idx		削除するスケジュールのインデックス
	* @return	削除されたスケジュール
	*/
	public Schedule removeSchedule( String name, Integer idx ) throws Exception {
		IDaemon dmn = manager.getDaemon( name );
		if ( dmn == null ) {
			throw( new IllegalArgumentException( "[" + name + "]"
				+ " not found." ));
		}
		if ( ! ( dmn instanceof CronDaemon )) {
			throw( new IllegalArgumentException( "[" + name + "]"
				+ " is not CronDaemon" ));
		}
		return ((CronDaemon)dmn).removeSchedule( idx.intValue() );
	}

	/**
	* TimerDaemonに間隔を追加します。
	* @param	name	デーモンの名前
	* @param	itv		間隔
	*/
	public void addInterval( String name, Interval itv ) throws Exception {
		IDaemon dmn = manager.getDaemon( name );
		if ( dmn == null ) {
			throw( new IllegalArgumentException( "[" + name + "]"
				+ " not found." ));
		}
		if ( ! ( dmn instanceof TimerDaemon )) {
			throw( new IllegalArgumentException( "[" + name + "]"
				+ " is not TimerDaemon" ));
		}
		((TimerDaemon)dmn).addInterval( itv );
	}

	/**
	* TimerDaemonの間隔を削除します。
	* @param	name	デーモンの名前
	* @param	idx		削除するスケジュールのインデックス
	* @return	削除されたスケジュール
	*/
	public Interval removeInterval( String name, Integer idx ) throws Exception {
		IDaemon dmn = manager.getDaemon( name );
		if ( dmn == null ) {
			throw( new IllegalArgumentException( "[" + name + "]"
				+ " not found." ));
		}
		if ( ! ( dmn instanceof TimerDaemon )) {
			throw( new IllegalArgumentException( "[" + name + "]"
				+ " is not TimerDaemon" ));
		}
		return ((TimerDaemon)dmn).removeInterval( idx.intValue() );
	}

	/**
	* CronDaemon,TimerDaemonの処理を直ちに実行します。
	* @param	name	デーモンの名前
	*/
	public void startImmediate( String name ) throws Exception {
		IDaemon dmn = manager.getDaemon( name );
		if ( dmn == null ) {
			throw( new IllegalArgumentException( "Daemon[" + name + "]"
				+ " not found." ));
		}
		if ( dmn instanceof CronDaemon ) {
			((CronDaemon)dmn).startImmediate();
		} else
		if ( dmn instanceof TimerDaemon ) {
			((TimerDaemon)dmn).startImmediate();
		} else {
			throw( new IllegalArgumentException( "Daemon[" + name + "]"
				+ " is not CronDaemon/TimerDaemon" ));
		}
	}

	/**
	* helpを表示します。
	*/
	public String help() {
		String ret = "";
		ret += "--- control command ---\n";
		ret += "start, stop, quit, reload, initialize\n";
		ret += "start(daemon), stop(daemon)], initialize(daemon)\n";
		ret += "save(file,root name)\n";

		ret += "--- information command ---\n";
		ret += "status, thread, vminfo, properties, gc, help\n";

		ret += "--- programable command ---\n";
		ret += "getServiceMap, getDaemonMap\n";
		ret += "getDaemon(daemon), removeDaemon(daemon), addDaemon(config)\n";
		ret += "addSchedule(name,schedule),removeSchedule(name,index),\n";
		ret += "addInterval(name.interval),removeInterval(name,index),\n";
		ret += "startImmediate(name)\n";
		return ret;
	}
}
