package	jp.co.sds_corp.util.daemon.remote.socket;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.remote.AbstractServer;
import jp.co.sds_corp.util.daemon.remote.Commander;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* デーモンマネージャのSocket I/F のサーバです。
<pre>
&lt;Remote type="Socket"&gt;
	&lt;host&gt;localhost&lt;/host&gt;
	&lt;port&gt;2011&lt;/port&gt;
&lt;/Remote&gt;
</pre>
*/
public class SocketServer extends AbstractServer
{
	static CmnLogger logger = CmnLogger.getInstance();
	String	host;
	int		nPort;
	ServerSocket srv = null;
	Thread	thr;
	boolean	bBreak = false;
	Commander	cmd;
	Vector<Restrict>	vecAllow = new Vector<Restrict>();
	Vector<Restrict>	vecDeny = new Vector<Restrict>();

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		setType( "Socket" );
		cmd = new Commander( getManager() );

		host = config.get( "host", null );
		nPort = Integer.valueOf( config.get( "port" )).intValue();
		logger.debug( this, "start SocketServer "
						+ "host=[" + host + "] port=[" + nPort + "]" );
		if ( host != null ) {
			InetAddress addr = InetAddress.getByName( host );
			srv = new ServerSocket( nPort, 50, addr );
		} else {
			srv = new ServerSocket( nPort );
		}
		srv.setSoTimeout( 1000 );

		Iterator<IConfiguration> itr;
		itr = config.getChilds( "Allow" );
		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			vecAllow.add( createRestrict( c, true ));
		}
		itr = config.getChilds( "Deny" );
		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			vecDeny.add( createRestrict( c, false ));
		}

		thr = new Thread( new ServerThread() );
		thr.setName( "SocketServer" );
		thr.start();
	}

	protected Restrict createRestrict( IConfiguration config, boolean allow )
		throws Exception
	{
		String from = config.getAttributes().get( "from" );
		Restrict ret = new Restrict( from, allow );
		return ret;
	}

	protected class ServerThread implements Runnable {
		public void run() {
			for ( ;; ) {
				if ( isBreak()) {
					break;
				}
				Thread.yield();
				try {
					Socket sock = srv.accept();
					if ( !isAccept( sock )) {
						continue;
					}
					ReceiverThread rec = new ReceiverThread( sock );
					rec.start();
				} catch ( SocketTimeoutException ex ) {
				} catch ( Exception ex ) {
					logger.warn( this, "accept error", ex );
				}
			}
			try {
				srv.close();
			} catch ( Exception ex ) {
				logger.error( this, "close error", ex );
			}
		}
	}

	protected boolean isAccept( Socket sock ) {
		InetAddress addr = ((InetSocketAddress )sock.getRemoteSocketAddress())
								.getAddress();
		boolean bRet = true;
		if ( vecAllow.size() > 0 ) {
			bRet = false;
			for ( int i = 0; i < vecAllow.size(); i++ ) {
				Restrict r = (Restrict )vecAllow.get( i );
				if ( r.match( addr )) {
					bRet = true;
					break;
				}
			}
		}
		if ( vecDeny.size() > 0 ) {
			for ( int i = 0; i < vecDeny.size(); i++ ) {
				Restrict r = (Restrict )vecDeny.get( i );
				if ( r.match( addr )) {
					bRet = false;
					break;
				}
			}
		}
		if ( isDebug()) {
			logger.debug( this, "accept from [" + addr + "] "
								+ ( bRet ? "allow" : "deny" ));
		}
		return bRet;
	}

	protected class ReceiverThread extends Thread {
		Socket sock;
		protected ReceiverThread( Socket sock ) {
			this.sock = sock;
			setName( "SocketServer:ReceiverThread" );
		}
		public void run() {
			try {
				receive( sock );
			} finally {
				try {
					sock.close();
				} catch ( Exception ex ) {
				}
			}
		}
	}

	@SuppressWarnings( "unchecked" )
	protected void receive( Socket sock ) {
		try {
			ObjectInputStream is = new ObjectInputStream(
				new BufferedInputStream( sock.getInputStream() ));
			Map<String,Object> reqMap = (Map<String,Object> )is.readObject();
			Object ret = command( reqMap );

			ObjectOutputStream os = new ObjectOutputStream(
				new BufferedOutputStream( sock.getOutputStream() ));
			os.writeObject( ret );
			os.flush();

			os.close();
			is.close();
		} catch ( Exception ex ) {
			logger.warn( this, "receive error", ex );
		}
	}

	protected Object command( Map<String,Object> map ) {
		Object ret = null;
		try {
			String method = (String )map.get( "command" );
			Class<?>[] cls = (Class<?>[] )map.get( "class" );
			Object[] prm = (Object[] )map.get( "param" );
			ret = super.command( cmd, method, cls, prm );
		} catch ( Exception ex ) {
			if ( isDebug()) {
				logger.warn( this, "command invoke error", ex );
			}
			ret = ex;
		}
		return ret;
	}

	protected synchronized void setBreak( boolean var ) {
		this.bBreak = var;
	}
	protected synchronized boolean isBreak() {
		return this.bBreak;
	}

	/**
	* ソケットサーバを終了します。
	*/
	public void exit() {
		setBreak( true );
		logger.debug( this, "stop SocketServer "
						+ "host=[" + host + "] port=[" + nPort + "]" );
		if ( thr != null ) {
			try {
				thr.join();
			} catch ( Exception ex ) {
				logger.warn( this, "join error", ex );
			}
		}
	}

}
